//*****************************************************************************
//
//	CAN - Interface basierend auf SPI-Bus mit MCP2510
//
//*****************************************************************************
#include <iom128v.h>
#include "m128int.h"
#include <stdio.h>
#include "canreg.h"
#include "can.h"
#include "spibus.h"
#include "watchdog.h"
#include "proto0.h"

#pragma interrupt_handler CAN_INT:EXT_INT7

#define CANNACHRLAENGE 10
CANMSG CANNACHR[CANNACHRLAENGE+1];
unsigned char uc_CANNACHRCOUNT = 0,uc_CANNACHRWR = 0,uc_CANNACHRRD = 0;
unsigned char uc_EXTENDED = 0;


//*****************************************************************************
//	CAN - Init
//  Input:  -	
//	Output:	-
//*****************************************************************************
void CAN_INIT(void)
{  
  DDRE  &= 0x7F;
  PORTE |= 0x80;
  EIFR   = 0xFF;
  EICRB &= 0x3F;  //fallende Flanke   Bit 6,7
  EIMSK |= 0x80;  //ext.int freigeben
  SREG  |= 0x80;  //INT Freigabe global

  SPI_Init(SPI_MASTER | SPI_CLK_DIV128);  
  CAN_RESET(); //SW
}


//*****************************************************************************
//	CAN - Interrupt (EXTINT 7)
//  Input:  -	
//	Output:	-
//*****************************************************************************
void CAN_INT(void)
{
unsigned char uc_puffer;
CANMSG *   pCANRXMSG;  

   uc_puffer = CAN_READREG(CANREG_CANINTF) & 0x03;
   
   if (uc_puffer)
    {
	   if ((uc_puffer & RX0IF)== RX0IF)
	    {
          pCANRXMSG = &CANNACHR[uc_CANNACHRWR]; 
		  if (uc_EXTENDED) pCANRXMSG->uc_EXTID = 1; 
		              else pCANRXMSG->uc_EXTID = 0;    
  	      CAN_READMSG(pCANRXMSG,0);
          uc_CANNACHRWR++;
	      uc_CANNACHRCOUNT++;
    	  if (uc_CANNACHRWR > CANNACHRLAENGE) uc_CANNACHRWR = 0;
		}  					

       if ((uc_puffer & RX1IF) == RX1IF)
	    {
          pCANRXMSG = &CANNACHR[uc_CANNACHRWR];    
		  if (uc_EXTENDED) pCANRXMSG->uc_EXTID = 1; 
		              else pCANRXMSG->uc_EXTID = 0;    
  	      CAN_READMSG(pCANRXMSG,1);
          uc_CANNACHRWR++;
	      uc_CANNACHRCOUNT++;
    	  if (uc_CANNACHRWR > CANNACHRLAENGE) uc_CANNACHRWR = 0;
		}			
   }  

   CAN_WRITEREG(CANREG_CANINTF,0x00);  //INTFLAGS zuruecksetzen 
}


//*****************************************************************************
//	CAN - Chipselect
//  Input:  1 aktiv
//	Output:	-
//*****************************************************************************
void CAN_CS(unsigned char uc_cs)
{
   if (uc_cs) PORTB &= 0xFE;
        else  PORTB |= 0x01;
}


//*****************************************************************************
//	CAN -  Software Reset
//  Input:  -
//	Output:	-
//*****************************************************************************
void CAN_RESET(void)
{
  CAN_CS(1);
  SPI_Write(0xC0);
  CAN_CS(0); 
}


//*****************************************************************************
//	CAN - Write Register 
//  Input:  Registeradresse,Daten
//	Output:	-
//*****************************************************************************
void CAN_WRITEREG(unsigned char uc_adr, unsigned char uc_data)
{
  CAN_CS(1);
  SPI_Write(0x02);
  SPI_Write(uc_adr);
  SPI_Write(uc_data);
  CAN_CS(0); 
}


//*****************************************************************************
//	CAN - Read Register 
//  Input:  Registeradresse
//	Output:	Daten
//*****************************************************************************
unsigned char CAN_READREG(unsigned char uc_adr)
{
unsigned char uc_data =0;

  CAN_CS(1);
  SPI_Write(0x03);
  SPI_Write(uc_adr);
  uc_data = SPI_Read(0x00);
  CAN_CS(0); 
  return(uc_data);
}


//*****************************************************************************
//	CAN - Read/Write RTS 
//  Input:  Daten
//	Output:	Daten
//*****************************************************************************
unsigned char CAN_RTS(unsigned char uc_data)
{
  CAN_CS(1);
  SPI_Write(0x80 | (uc_data & 0x07));
  CAN_CS(0); 
  return(SPI_ReadData());
}


//*****************************************************************************
//	CAN - Read Status 
//  Input:  Registeradresse
//	Output:	Daten
//*****************************************************************************
unsigned char CAN_READSTATUSREG(void)
{
unsigned char uc_data =0;

  CAN_CS(1);
  SPI_Write(0xA0);
  uc_data = SPI_Read(0x00);
  SPI_Read(0x00);
  CAN_CS(0); 
  return(uc_data);
}


//*****************************************************************************
//	CAN - Bitmodify 
//  Input:  Registeradresse,Bit,Maske
//	Output:	Daten
//*****************************************************************************
void CAN_REGBIT(unsigned char uc_adr,unsigned char uc_bit,unsigned char uc_mask)
{
   CAN_CS(1);
   SPI_Write(0x05);
   SPI_Write(uc_adr);
   SPI_Write(uc_mask);
   SPI_Write(uc_bit);
   CAN_CS(0);    
}


//*****************************************************************************
//	CAN - OPEN 
//  Input:  CANBAUD
//	Output:	Bool 0/1   1 ok  0 Error
//*****************************************************************************
unsigned char CAN_OPEN(CANBAUD can_baud,unsigned char uc_MSGTYPE)
{
unsigned char CNF1,CNF2,CNF3;

   uc_CANNACHRCOUNT = 0;
   uc_CANNACHRWR = 0;
   uc_CANNACHRRD = 0;
   uc_EXTENDED = uc_MSGTYPE;
   
   CAN_INIT();
   CAN_WRITEREG(CANREG_CTRL,CAN_CONFIGMODE);

   switch(can_baud)
   {
   case CAN_BAUD_1M:       //   1 MBit/s 00 90 02
                      CNF1 = 0x40;
                      CNF2 = 0x10;
                      CNF3 = 0x01;					  
                      break;
   case CAN_BAUD_500K:     // 500 kBit/s 00 B8 05
                      CNF1 = 0x40;
                      CNF2 = 0x2A;
                      CNF3 = 0x01;					  
                      break;
   case CAN_BAUD_250K:     // 250 kBit/s 01 B8 05
                      CNF1 = 0x41;
                      CNF2 = 0x2A;
                      CNF3 = 0x01;				  
                      break;
   case CAN_BAUD_125K:     // 125 kBit/s 03 B8 05 
                      CNF1 = 0x43;
                      CNF2 = 0x2A;
                      CNF3 = 0x01;					  
                      break;
   case CAN_BAUD_100K:     // 100 kBit/s 03 BA 07
                      CNF1 = 0x44;
                      CNF2 = 0x2A;
                      CNF3 = 0x01;			  
                      break;
   case CAN_BAUD_50K:      //  50 kBit/s 07 BA 07
                      CNF1 = 0x49;
                      CNF2 = 0x2A;
                      CNF3 = 0x01;			  
                      break;
   case CAN_BAUD_20K:      //  20 kBit/s 0F BF 07
                      CNF1 = 0x58;
                      CNF2 = 0x2A;
                      CNF3 = 0x01;			  
                      break;
   case CAN_BAUD_10K:      //  10 kBit/s 1F BF 07
                      CNF1 = 0x71;
                      CNF2 = 0x2A;
                      CNF3 = 0x01;			  
                      break;
   case CAN_BAUD_5K:       //   5 kBit/s 3F B7 07
                      CNF1 = 0x7F;
                      CNF2 = 0x3F;
                      CNF3 = 0x01;			  
                      break;
   }
   
   CAN_WRITEREG(CANREG_CNF1,CNF1);            //Konfiguration, Baudrate laden
   CAN_WRITEREG(CANREG_CNF2,CNF2);
   CAN_WRITEREG(CANREG_CNF3,CNF3);
   CAN_WRITEREG(CANREG_EFLG,0x00);            //Overflow-Flags ruecksetzen 
   CAN_WRITEREG(CANREG_RXB0CTRL,0x00);        //alle gueltigen MSG empfangen
   CAN_WRITEREG(CANREG_RXB1CTRL,0x00);
   CAN_WRITEREG(CANREG_CANINTE,RX0IE | RX1IE);
   CAN_WRITEREG(CANREG_CTRL,CAN_NORMALMODE);  //und los gehts 
   return(0);
}


//*****************************************************************************
//	CAN - Close 
//  Input:  -
//	Output:	-
//*****************************************************************************
void CAN_CLOSE(void)
{
   CAN_RESET();
}


//*****************************************************************************
//	CAN - SENDMSG 
//  Input:  CANADRESSE,PCANMSG
//	Output:	Status
//*****************************************************************************
unsigned char CAN_SENDMSG(CANMSG * pCANMSG,unsigned char uc_puffer)
{
unsigned char uc_status;

  switch(uc_puffer)
  {
  case 0:
          while((CAN_READREG(CANREG_TXB0CTRL) & 0x08) == TXREQ)
          {
             watchdog_reset();  
          }

          CAN_REG_TXB0(pCANMSG);
          CAN_WRITEREG(CANREG_TXB0CTRL,0x08);
          uc_status = CAN_READREG(CANREG_TXB0CTRL);
		  break;
		  
  case 1:
          while((CAN_READREG(CANREG_TXB1CTRL) & 0x08) == TXREQ)
          {
             watchdog_reset();  
          }

          CAN_REG_TXB1(pCANMSG);
          CAN_WRITEREG(CANREG_TXB1CTRL,0x08);
          uc_status = CAN_READREG(CANREG_TXB1CTRL);
		  break;
  case 2:
          while((CAN_READREG(CANREG_TXB2CTRL) & 0x08) == TXREQ)
          {
             watchdog_reset();  
          }

          CAN_REG_TXB2(pCANMSG);
          CAN_WRITEREG(CANREG_TXB2CTRL,0x08);
          uc_status = CAN_READREG(CANREG_TXB2CTRL);
		  break;
  }
  		  
  if ((uc_status & 0x10) == 0x10) return(CAN_ERR_BUSHEAVY);
  if ((uc_status & 0x20) == 0x20) return(CAN_ERR_LOST);
  if ((uc_status & 0x40) == 0x40) return(CAN_ERR_ABORT); 
  
  return(CAN_STATUS());
}


//*****************************************************************************
//	CAN - READMSG 
//  Input:  CANADRESSE,PCANMSG
//	Output:	Status
//*****************************************************************************
unsigned char CAN_READMSG(CANMSG * pCANMSG,unsigned char uc_puffer)
{
   switch(uc_puffer)
   {
   case 0: CAN_REG_RXB0(pCANMSG);
           CAN_REGBIT(CANREG_RXB0CTRL,0x00,0x01);
           break;
   case 1: CAN_REG_RXB1(pCANMSG);
           CAN_REGBIT(CANREG_RXB0CTRL,0x00,0x02);   
           break;
   }
   return(CAN_STATUS());
}


//*****************************************************************************
//	CAN - RX Test 
//  Input:  -
//	Output:	Status 0/1   0 keine, >0 neue Nachricht im Puffer 
//*****************************************************************************
unsigned char CAN_RXTEST(void)
{
   return(uc_CANNACHRCOUNT);
}


//*****************************************************************************
//	CAN - RECEIVE 
//  Input:  -
//	Output:	Pointer auf CANMSG im Puffer 
//*****************************************************************************
CANMSG * CAN_RECEIVE(CANMSG * pCANMSG)
{
   pCANMSG = &CANNACHR[uc_CANNACHRRD]; 
   uc_CANNACHRRD++;
   if (uc_CANNACHRRD > CANNACHRLAENGE) uc_CANNACHRRD = 0;
   if (uc_CANNACHRCOUNT > 0) uc_CANNACHRCOUNT--;
   return(pCANMSG);  
}


//*****************************************************************************
//	CAN - STATUS 
//  Input:  -
//	Output:	CAN-Status 
//*****************************************************************************
unsigned char CAN_STATUS(void)
{ 
unsigned char uc_status;
   
   uc_status = CAN_READREG(CANREG_EFLG);

   if ((uc_status & 0x20) == TXBO) return(CAN_ERR_BUSOFF);   
   if ((uc_status & 0x18) == TXBO) return(CAN_ERR_BUSHEAVY);
   return(CAN_ERR_OK);
}


//*****************************************************************************
//	CAN - REG_RXM0 
//  Input:  CANADRESSE  extended Format Maske
//	Output:	-
//*****************************************************************************
void CAN_REG_RXM0(CANADR can_adr,unsigned char uc_EXTID)
{
unsigned char DB[4];
unsigned long ul_adr;

  if (uc_EXTID) ul_adr = can_adr;
           else ul_adr = can_adr << 18;
   
   DB[0] = (unsigned char) ul_adr;
   DB[1] = (unsigned long)  ul_adr >> 8;
   DB[2] = ((unsigned long) (ul_adr >> 16) & 0x03) | 
           ((unsigned long) (ul_adr >> 13) & 0xE0);
   DB[3] = (unsigned long) ul_adr >> 21;   

   CAN_WRITEREG(CANREG_RXM0EID0,DB[0]);
   CAN_WRITEREG(CANREG_RXM0EID8,DB[1]);
   CAN_WRITEREG(CANREG_RXM0SIDL,DB[2]);
   CAN_WRITEREG(CANREG_RXM0SIDH,DB[3]);
   if (uc_EXTID == ID_EXTENDED) CAN_WRITEREG(CANREG_RXB0CTRL,0x02);
                           else CAN_WRITEREG(CANREG_RXB0CTRL,0x01);
}


//*****************************************************************************
//	CAN - REG_RXM1 
//  Input:  CANADRESSE  extended Format Maske
//	Output:	-
//*****************************************************************************
void CAN_REG_RXM1(CANADR can_adr,unsigned char uc_EXTID)
{
unsigned char DB[4];
unsigned long ul_adr;

  if (uc_EXTID) ul_adr = can_adr;
           else ul_adr = can_adr << 18;
   
   DB[0] = (unsigned char) ul_adr;
   DB[1] = (unsigned long)  ul_adr >> 8;
   DB[2] = ((unsigned long) (ul_adr >> 16) & 0x03) | 
           ((unsigned long) (ul_adr >> 13) & 0xE0);
   DB[3] = (unsigned long) ul_adr >> 21;   

   CAN_WRITEREG(CANREG_RXM1EID0,DB[0]);
   CAN_WRITEREG(CANREG_RXM1EID8,DB[1]);
   CAN_WRITEREG(CANREG_RXM1SIDL,DB[2]);
   CAN_WRITEREG(CANREG_RXM1SIDH,DB[3]);
   if (uc_EXTID == ID_EXTENDED) CAN_WRITEREG(CANREG_RXB1CTRL,0x02);
                           else CAN_WRITEREG(CANREG_RXB1CTRL,0x01);
}


//*****************************************************************************
//	CAN - REG_RXF0 
//  Input:  CANADRESSE  extended Format Filter
//	Output:	-
//*****************************************************************************
void CAN_REG_RXF0(CANADR can_adr,unsigned char uc_EXTID)
{
unsigned char DB[4];
unsigned long ul_adr;

  if (uc_EXTID) ul_adr = can_adr;
           else ul_adr = can_adr << 18;
   
   DB[0] = (unsigned char) ul_adr;
   DB[1] = (unsigned long)  ul_adr >> 8;
   DB[2] = ((unsigned long) (ul_adr >> 16) & 0x03) | 
           ((unsigned long) (ul_adr >> 13) & 0xE0);
   DB[3] = (unsigned long) ul_adr >> 21;   

   CAN_WRITEREG(CANREG_RXF0EID0,DB[0]);
   CAN_WRITEREG(CANREG_RXF0EID8,DB[1]);
   CAN_WRITEREG(CANREG_RXF0SIDL,DB[2]);
   CAN_WRITEREG(CANREG_RXF0SIDH,DB[3]);
   if (uc_EXTID == ID_EXTENDED) CAN_WRITEREG(CANREG_RXB0CTRL,0x02);
                           else CAN_WRITEREG(CANREG_RXB0CTRL,0x01);
}


//*****************************************************************************
//	CAN - REG_RXF1 
//  Input:  CANADRESSE  extended Format Filter
//	Output:	-
//*****************************************************************************
void CAN_REG_RXF1(CANADR can_adr,unsigned char uc_EXTID)
{
unsigned char DB[4];
unsigned long ul_adr;

  if (uc_EXTID) ul_adr = can_adr;
           else ul_adr = can_adr << 18;
   
   DB[0] = (unsigned char) ul_adr;
   DB[1] = (unsigned long)  ul_adr >> 8;
   DB[2] = ((unsigned long) (ul_adr >> 16) & 0x03) | 
           ((unsigned long) (ul_adr >> 13) & 0xE0);
   DB[3] = (unsigned long) ul_adr >> 21;   

   CAN_WRITEREG(CANREG_RXF1EID0,DB[0]);
   CAN_WRITEREG(CANREG_RXF1EID8,DB[1]);
   CAN_WRITEREG(CANREG_RXF1SIDL,DB[2]);
   CAN_WRITEREG(CANREG_RXF1SIDH,DB[3]);
   if (uc_EXTID == ID_EXTENDED) CAN_WRITEREG(CANREG_RXB0CTRL,0x02);
                           else CAN_WRITEREG(CANREG_RXB0CTRL,0x01);
}


//*****************************************************************************
//	CAN - REG_RXF2 
//  Input:  CANADRESSE  extended Format Filter
//	Output:	-
//*****************************************************************************
void CAN_REG_RXF2(CANADR can_adr,unsigned char uc_EXTID)
{
unsigned char DB[4];
unsigned long ul_adr;

  if (uc_EXTID) ul_adr = can_adr;
           else ul_adr = can_adr << 18;
   
   DB[0] = (unsigned char) ul_adr;
   DB[1] = (unsigned long)  ul_adr >> 8;
   DB[2] = ((unsigned long) (ul_adr >> 16) & 0x03) | 
           ((unsigned long) (ul_adr >> 13) & 0xE0);
   DB[3] = (unsigned long) ul_adr >> 21;   

   CAN_WRITEREG(CANREG_RXF2EID0,DB[0]);
   CAN_WRITEREG(CANREG_RXF2EID8,DB[1]);
   CAN_WRITEREG(CANREG_RXF2SIDL,DB[2]);
   CAN_WRITEREG(CANREG_RXF2SIDH,DB[3]);
   if (uc_EXTID == ID_EXTENDED) CAN_WRITEREG(CANREG_RXB1CTRL,0x02);
                           else CAN_WRITEREG(CANREG_RXB1CTRL,0x01);
}


//*****************************************************************************
//	CAN - REG_RXF3 
//  Input:  CANADRESSE  extended Format Filter
//	Output:	-
//*****************************************************************************
void CAN_REG_RXF3(CANADR can_adr,unsigned char uc_EXTID)
{
unsigned char DB[4];
unsigned long ul_adr;

  if (uc_EXTID) ul_adr = can_adr;
           else ul_adr = can_adr << 18;
   
   DB[0] = (unsigned char) ul_adr;
   DB[1] = (unsigned long)  ul_adr >> 8;
   DB[2] = ((unsigned long) (ul_adr >> 16) & 0x03) | 
           ((unsigned long) (ul_adr >> 13) & 0xE0);
   DB[3] = (unsigned long) ul_adr >> 21;   

   CAN_WRITEREG(CANREG_RXF3EID0,DB[0]);
   CAN_WRITEREG(CANREG_RXF3EID8,DB[1]);
   CAN_WRITEREG(CANREG_RXF3SIDL,DB[2]);
   CAN_WRITEREG(CANREG_RXF3SIDH,DB[3]);
   if (uc_EXTID == ID_EXTENDED) CAN_WRITEREG(CANREG_RXB1CTRL,0x02);
                           else CAN_WRITEREG(CANREG_RXB1CTRL,0x01);
}


//*****************************************************************************
//	CAN - REG_RXF4 
//  Input:  CANADRESSE  extended Format Filter
//	Output:	-
//*****************************************************************************
void CAN_REG_RXF4(CANADR can_adr,unsigned char uc_EXTID)
{
unsigned char DB[4];
unsigned long ul_adr;

  if (uc_EXTID) ul_adr = can_adr;
           else ul_adr = can_adr << 18;
   
   DB[0] = (unsigned char) ul_adr;
   DB[1] = (unsigned long)  ul_adr >> 8;
   DB[2] = ((unsigned long) (ul_adr >> 16) & 0x03) | 
           ((unsigned long) (ul_adr >> 13) & 0xE0);
   DB[3] = (unsigned long) ul_adr >> 21;   

   CAN_WRITEREG(CANREG_RXF4EID0,DB[0]);
   CAN_WRITEREG(CANREG_RXF4EID8,DB[1]);
   CAN_WRITEREG(CANREG_RXF4SIDL,DB[2]);
   CAN_WRITEREG(CANREG_RXF4SIDH,DB[3]);
   if (uc_EXTID == ID_EXTENDED) CAN_WRITEREG(CANREG_RXB1CTRL,0x02);
                           else CAN_WRITEREG(CANREG_RXB1CTRL,0x01);
}


//*****************************************************************************
//	CAN - REG_RXF5 
//  Input:  CANADRESSE  extended Format Filter
//	Output:	-
//*****************************************************************************
void CAN_REG_RXF5(CANADR can_adr,unsigned char uc_EXTID)
{
unsigned char DB[4];
unsigned long ul_adr;

  if (uc_EXTID) ul_adr = can_adr;
           else ul_adr = can_adr << 18;
   
   DB[0] = (unsigned char) ul_adr;
   DB[1] = (unsigned long)  ul_adr >> 8;
   DB[2] = ((unsigned long) (ul_adr >> 16) & 0x03) | 
           ((unsigned long) (ul_adr >> 13) & 0xE0);
   DB[3] = (unsigned long) ul_adr >> 21;   

   CAN_WRITEREG(CANREG_RXF5EID0,DB[0]);
   CAN_WRITEREG(CANREG_RXF5EID8,DB[1]);
   CAN_WRITEREG(CANREG_RXF5SIDL,DB[2]);
   CAN_WRITEREG(CANREG_RXF5SIDH,DB[3]);
   if (uc_EXTID == ID_EXTENDED) CAN_WRITEREG(CANREG_RXB1CTRL,0x02);
                           else CAN_WRITEREG(CANREG_RXB1CTRL,0x01);
}


//*****************************************************************************
//	CAN - REG_TXB0
//  Input:  CANMESSAGE
//	Output:	-
//*****************************************************************************
void CAN_REG_TXB0(CANMSG * pCANMSG)
{
unsigned char DB[4];
unsigned long ul_adr;

  if (pCANMSG->uc_EXTID) ul_adr = pCANMSG->can_ADR;
                    else ul_adr = pCANMSG->can_ADR << 18;
			
   
   DB[0] = (unsigned char) ul_adr;
   DB[1] = (unsigned long)  ul_adr >> 8;
   DB[2] = ((unsigned long) (ul_adr >> 16) & 0x03) | 
           ((unsigned long) (ul_adr >> 13) & 0xE0);
   DB[3] = (unsigned long) ul_adr >> 21;   

  if (pCANMSG->uc_EXTID) DB[2] |= 0x08;   
      
  CAN_CS(1);
  SPI_Write(0x02);
  SPI_Write(CANREG_TXB0SIDH);
  SPI_Write(DB[3]);
  SPI_Write(DB[2]);
  SPI_Write(DB[1]);
  SPI_Write(DB[0]);
  if (pCANMSG->uc_TYPE) SPI_Write((pCANMSG->uc_len & 0x0F) | 0x40); 
                   else SPI_Write(pCANMSG->uc_len & 0x0F);
  SPI_Write(pCANMSG->uc_DB0);				  
  SPI_Write(pCANMSG->uc_DB1);				  
  SPI_Write(pCANMSG->uc_DB2);				  
  SPI_Write(pCANMSG->uc_DB3);				  
  SPI_Write(pCANMSG->uc_DB4);				  
  SPI_Write(pCANMSG->uc_DB5);				  
  SPI_Write(pCANMSG->uc_DB6);				  
  SPI_Write(pCANMSG->uc_DB7);				  			  
  CAN_CS(0); 
}


//*****************************************************************************
//	CAN - REG_TXB1
//  Input:  CANMESSAGE
//	Output:	-
//*****************************************************************************
void CAN_REG_TXB1(CANMSG * pCANMSG)
{
unsigned char DB[4];
unsigned long ul_adr;

  if (pCANMSG->uc_EXTID) ul_adr = pCANMSG->can_ADR;
                    else ul_adr = pCANMSG->can_ADR << 18;
			
   
   DB[0] = (unsigned char) ul_adr;
   DB[1] = (unsigned long)  ul_adr >> 8;
   DB[2] = ((unsigned long) (ul_adr >> 16) & 0x03) | 
           ((unsigned long) (ul_adr >> 13) & 0xE0);
   DB[3] = (unsigned long) ul_adr >> 21;   

  if (pCANMSG->uc_EXTID) DB[2] |= 0x08;   

  CAN_CS(1);
  SPI_Write(0x02);
  SPI_Write(CANREG_TXB1SIDH);
  SPI_Write(DB[3]);
  SPI_Write(DB[2]);
  SPI_Write(DB[1]);
  SPI_Write(DB[0]);
  if (pCANMSG->uc_TYPE) SPI_Write((pCANMSG->uc_len & 0x0F) | 0x40); 
                   else SPI_Write(pCANMSG->uc_len & 0x0F);
  SPI_Write(pCANMSG->uc_DB0);				  
  SPI_Write(pCANMSG->uc_DB1);				  
  SPI_Write(pCANMSG->uc_DB2);				  
  SPI_Write(pCANMSG->uc_DB3);				  
  SPI_Write(pCANMSG->uc_DB4);				  
  SPI_Write(pCANMSG->uc_DB5);				  
  SPI_Write(pCANMSG->uc_DB6);				  
  SPI_Write(pCANMSG->uc_DB7);				  			  
  CAN_CS(0); 
}


//*****************************************************************************
//	CAN - REG_TXB2
//  Input:  CANMESSAGE
//	Output:	-
//*****************************************************************************
void CAN_REG_TXB2(CANMSG * pCANMSG)
{
unsigned char DB[4];
unsigned long ul_adr;

  if (pCANMSG->uc_EXTID) ul_adr = pCANMSG->can_ADR;
                    else ul_adr = pCANMSG->can_ADR << 18;
			
   
   DB[0] = (unsigned char) ul_adr;
   DB[1] = (unsigned long)  ul_adr >> 8;
   DB[2] = ((unsigned long) (ul_adr >> 16) & 0x03) | 
           ((unsigned long) (ul_adr >> 13) & 0xE0);
   DB[3] = (unsigned long) ul_adr >> 21;   

  if (pCANMSG->uc_EXTID) DB[2] |= 0x08;   

  CAN_CS(1);
  SPI_Write(0x02);
  SPI_Write(CANREG_TXB2SIDH);
  SPI_Write(DB[3]);
  if (pCANMSG->uc_EXTID) SPI_Write((DB[2] & 0x03) | (DB[0] << 5) | 0x08);
                    else SPI_Write((DB[2] & 0x03) | (DB[0] << 5));        
  SPI_Write(DB[1]);
  SPI_Write(DB[0]);
  if (pCANMSG->uc_TYPE) SPI_Write((pCANMSG->uc_len & 0x0F) | 0x40); 
                   else SPI_Write(pCANMSG->uc_len & 0x0F);
  SPI_Write(pCANMSG->uc_DB0);				  
  SPI_Write(pCANMSG->uc_DB1);				  
  SPI_Write(pCANMSG->uc_DB2);				  
  SPI_Write(pCANMSG->uc_DB3);				  
  SPI_Write(pCANMSG->uc_DB4);				  
  SPI_Write(pCANMSG->uc_DB5);				  
  SPI_Write(pCANMSG->uc_DB6);				  
  SPI_Write(pCANMSG->uc_DB7);				  			  
  CAN_CS(0); 
}


//*****************************************************************************
//	CAN - REG_RXB0
//  Input:  CANMESSAGE
//	Output:	-
//*****************************************************************************
void CAN_REG_RXB0(CANMSG * pCANMSG)
{
unsigned char DB[4];
unsigned long ul_adr;

  CAN_CS(1);
  SPI_Write(0x03);
  SPI_Write(CANREG_RXB0CTRL);
  if (SPI_Read(0x00) == 0x08) pCANMSG->uc_TYPE = MSGTYPE_RTR;
                         else pCANMSG->uc_TYPE = MSGTYPE_DATA;
  DB[3] = SPI_Read(0x00);
  DB[2] = SPI_Read(0x00);
  DB[1] = SPI_Read(0x00);
  DB[0] = SPI_Read(0x00);
  
  ul_adr = (((unsigned long) DB[3] << 21) & 0x1FE00000) |
           (((unsigned long) DB[2] << 13) & 0x001C0000) |
           (((unsigned long) DB[2] << 16) & 0x00030000) | 
		   (((unsigned long) DB[2] << 16) & 0x000000FF) |
            ((unsigned long) DB[1] << 8)                | 
			 (unsigned long) DB[0];

  if (!pCANMSG->uc_EXTID) ul_adr = ul_adr >> 18;

  pCANMSG->can_ADR = ul_adr; 
  pCANMSG->uc_len = SPI_Read(0x00);
  pCANMSG->uc_DB0 = SPI_Read(0x00);
  pCANMSG->uc_DB1 = SPI_Read(0x00);
  pCANMSG->uc_DB2 = SPI_Read(0x00);
  pCANMSG->uc_DB3 = SPI_Read(0x00);
  pCANMSG->uc_DB4 = SPI_Read(0x00);
  pCANMSG->uc_DB5 = SPI_Read(0x00);
  pCANMSG->uc_DB6 = SPI_Read(0x00);
  pCANMSG->uc_DB7 = SPI_Read(0x00);
  CAN_CS(0); 
}


//*****************************************************************************
//	CAN - REG_RXB1
//  Input:  CANMESSAGE
//	Output:	-
//*****************************************************************************
void CAN_REG_RXB1(CANMSG * pCANMSG)
{
unsigned char DB[4];
unsigned long ul_adr;

  CAN_CS(1);
  SPI_Write(0x03);
  SPI_Write(CANREG_RXB1CTRL);
  if (SPI_Read(0x00) == 0x08) pCANMSG->uc_TYPE = MSGTYPE_RTR;
                         else pCANMSG->uc_TYPE = MSGTYPE_DATA;
  DB[3] = SPI_Read(0x00);
  DB[2] = SPI_Read(0x00);
  DB[1] = SPI_Read(0x00);
  DB[0] = SPI_Read(0x00);

  ul_adr = (((unsigned long) DB[3] << 21) & 0x1FE00000) |
           (((unsigned long) DB[2] << 13) & 0x001C0000) |
           (((unsigned long) DB[2] << 16) & 0x00030000) | 
		   (((unsigned long) DB[2] << 16) & 0x000000FF) |
            ((unsigned long) DB[1] << 8)                | 
			 (unsigned long) DB[0];

  if (!pCANMSG->uc_EXTID) ul_adr = ul_adr >> 18;

  pCANMSG->can_ADR = ul_adr; 
  pCANMSG->uc_len = SPI_Read(0x00);
  pCANMSG->uc_DB0 = SPI_Read(0x00);
  pCANMSG->uc_DB1 = SPI_Read(0x00);
  pCANMSG->uc_DB2 = SPI_Read(0x00);
  pCANMSG->uc_DB3 = SPI_Read(0x00);
  pCANMSG->uc_DB4 = SPI_Read(0x00);
  pCANMSG->uc_DB5 = SPI_Read(0x00);
  pCANMSG->uc_DB6 = SPI_Read(0x00);
  pCANMSG->uc_DB7 = SPI_Read(0x00);
  CAN_CS(0); 
}

