//*****************************************************************************
//
//	CAN-BUS
//
//*****************************************************************************
#ifndef _CANBUS_H_
#define _CANBUS_H_

//Betriebsmodi
#define CAN_CONFIGMODE     0x80
#define CAN_NORMALMODE     0x00
#define CAN_SLEEPMODE      0x20
#define CAN_LOOPBACKMODE   0x40
#define CAN_LISTENMODE     0x60


// Baudratencodes = Registerwerte
#define CAN_BAUD_1M        0x00  //   1 MBit/s
#define CAN_BAUD_500K      0x01  // 500 kBit/s
#define CAN_BAUD_250K      0x02  // 250 kBit/s
#define CAN_BAUD_125K      0x03  // 125 kBit/s
#define CAN_BAUD_100K      0x04  // 100 kBit/s
#define CAN_BAUD_50K       0x05  //  50 kBit/s
#define CAN_BAUD_20K       0x06  //  20 kBit/s
#define CAN_BAUD_10K       0x07  //  10 kBit/s
#define CAN_BAUD_5K        0x08  //   5 kBit/s


// Msg Type:
#define CAN_INIT_TYPE_EX   0x01	 //Extended Frame
#define CAN_INIT_TYPE_ST   0x00	 //Standart Frame 

// Fehlerzustaende
#define CAN_ERR_OK         0x00
#define CAN_ERR_LOST       0x01  // Arbitration fehlgeschlagen
#define CAN_ERR_BUSHEAVY   0x02  // Busfehler: ein Errorcounter erreichte Limit
#define CAN_ERR_BUSOFF     0x04  // Busfehler: CAN_Controller ging 'Bus-Off'
#define CAN_ERR_ABORT      0x08  // Abbruch der Aktion

#define ID_EXTENDED        0x01  // 1, wenn CAN 2.0 B Frame (29 Bit ID)
#define MSGTYPE_RTR        0x01  // 1, wenn remote request, sonst Data Msg
#define MSGTYPE_DATA       0x00  // Data Msg


//*****************************************************************************
//	CAN TYPE 
//*****************************************************************************
typedef unsigned long    CANADR;
typedef unsigned char    CANBAUD;
typedef struct  {
		 		        CANADR         can_ADR;
						unsigned char  uc_EXTID;
						unsigned char  uc_TYPE;
						unsigned char  uc_len; 
                        unsigned char  uc_DB0;
                        unsigned char  uc_DB1;
                        unsigned char  uc_DB2;
                        unsigned char  uc_DB3;
                        unsigned char  uc_DB4;
                        unsigned char  uc_DB5;
                        unsigned char  uc_DB6;
                        unsigned char  uc_DB7;
        		} CANMSG;


//*****************************************************************************
//	CAN Interrupt
//*****************************************************************************
void CAN_INT(void);


//*****************************************************************************
//	CAN Funktion
//*****************************************************************************
unsigned char CAN_OPEN(CANBAUD can_baud,unsigned char uc_MSGTYPE);
void CAN_KNOTEN_CLOSE(void);
void CAN_RESET(void);
unsigned char CAN_SENDMSG(CANMSG * pCANMSG,unsigned char uc_puffer);
unsigned char CAN_READMSG(CANMSG * pCANMSG,unsigned char uc_puffer);
unsigned char CAN_RXTEST(void);
CANMSG * CAN_RECEIVE(CANMSG * pCANMSG);
unsigned char CAN_STATUS(void);

//*****************************************************************************
//	CAN Register Funktion des MCP2510
//*****************************************************************************
unsigned char CAN_RTS(unsigned char uc_data);
unsigned char CAN_READSTATUSREG(void);
void CAN_REG_RXM0(CANADR can_adr,unsigned char uc_EXTID);
void CAN_REG_RXM1(CANADR can_adr,unsigned char uc_EXTID);
void CAN_REG_RXF0(CANADR can_adr,unsigned char uc_EXTID);
void CAN_REG_RXF1(CANADR can_adr,unsigned char uc_EXTID);
void CAN_REG_RXF2(CANADR can_adr,unsigned char uc_EXTID);
void CAN_REG_RXF3(CANADR can_adr,unsigned char uc_EXTID);
void CAN_REG_RXF4(CANADR can_adr,unsigned char uc_EXTID);
void CAN_REG_RXF5(CANADR can_adr,unsigned char uc_EXTID);
void CAN_REG_TXB0(CANMSG * pCANMSG);
void CAN_REG_TXB1(CANMSG * pCANMSG);
void CAN_REG_TXB2(CANMSG * pCANMSG);
void CAN_REG_RXB0(CANMSG * pCANMSG);
void CAN_REG_RXB1(CANMSG * pCANMSG);

//*****************************************************************************
//	CAN interne Funktion
//*****************************************************************************
//Schreib-Lesefkt.ueber SPI allgemein
void CAN_INIT(void);
void CAN_CS(unsigned char uc_cs);
void CAN_WRITEREG(unsigned char uc_adr, unsigned char uc_data);
unsigned char CAN_READREG(unsigned char uc_adr);
void CAN_REGBIT(unsigned char uc_adr,unsigned char uc_bit,unsigned char uc_mask);



#endif

