//*****************************************************************************
//
//	I2C Slave   empfaengt nur Daten
//
//  EXT_Int 6 SCL
//  EXT_Int 7 SDA
//
//*****************************************************************************
#include <io8535.h>
#include <string.h>
#include <stdlib.h>
#include "8535int.h"
#include "i2cslave.h"


#define SLAVE_ADR      0x70
#define BUFFER_LAENGE  50

#pragma interrupt_handler EXTINT0_INT:EXT_INT0
#pragma interrupt_handler EXTINT1_INT:EXT_INT1

char c_RXBUFFER[BUFFER_LAENGE];
unsigned int ui_RXCNT;
unsigned char uc_bit, uc_byte;

struct {
unsigned   bit_CONSTART       : 1,
		   bit_CONSTOP        : 1,
		   bit_RECEIVE        : 1,
		   bit_ACKN           : 1;
} state; //bit_field



//*****************************************************************************
//	EXTINT0 - INT   SCL
//*****************************************************************************

void EXTINT0_INT(void)
{
   //SDA tristate
   DDRD  &= 0xF3;
   PORTD &= 0xF3; 

   if (state.bit_CONSTART) 
    {
	   state.bit_RECEIVE  = 1; //Datenuebertragung beginnt 
	   state.bit_CONSTART = 0; //alte States loeschen
	   ui_RXCNT = 0;
	   uc_bit = 0;
	   uc_byte = 0;
	} 

   if (state.bit_RECEIVE)
    {
	   //fuer Daten - Taktflanke des CLK-Signals auswerten
	   if (uc_bit < 8)
	    {
		  uc_byte = uc_byte << 1; 
 	      uc_byte |= i2cslave_readSDA();
 	      uc_bit++;
		}
		  
	   if (uc_bit == 9)  //Acknolege
	    {
		  uc_bit = 0;
		  if (state.bit_ACKN)
		   {
		      //SDA = L  setzen
			  DDRD  |= 0x08;   //Ausgang aktiv
			  PORTD &= 0xF7;   //L-Pegel setzen
  		   }
		}	

	   if (uc_bit == 8)  // ein Byte gelesen
	    {
		  uc_bit++;
		  c_RXBUFFER[ui_RXCNT] = uc_byte;  //Byte in Puffer schreiben
		  if (ui_RXCNT < BUFFER_LAENGE) ui_RXCNT++;
		}

	}
}


//*****************************************************************************
//	EXTINT1 - INT  SDA
//*****************************************************************************

void EXTINT1_INT(void)
{
   if (state.bit_CONSTART || state.bit_RECEIVE)
    {
	   if (i2cslave_readSCL())     //SCL = H
	    {
		   state.bit_CONSTOP  = 1; //CONSTOP erkannt
		   state.bit_CONSTART = 0; //alte States loeschen
		   state.bit_RECEIVE  = 0;
		   state.bit_ACKN     = 0;
		   MCUCR &= 0xFB;          //SDA L-trigger
		   c_RXBUFFER[ui_RXCNT] = '\0';
		} 
	}
   else {	
          if (!state.bit_CONSTART && !state.bit_RECEIVE)
           {
        	 if (i2cslave_readSCL())      //SCL = H
	          {
   		        state.bit_CONSTART = 1;  //CONSTART erkannt
		        state.bit_CONSTOP = 0;   //alte States loeschen
		        state.bit_RECEIVE = 0;
		        state.bit_ACKN    = 0;
		        ui_RXCNT = 0;
		        uc_bit = 0;
		        MCUCR |= 0x04;            //SDA H-trigger
		      }
		   }	     
	  }
}




void i2cslave_init(void)
{
   DDRD &= 0xF3;
   PORTD &= 0xF3;

   state.bit_CONSTART = 0;
   state.bit_CONSTOP  = 0;
   state.bit_RECEIVE  = 0;
   state.bit_ACKN     = 0;

   uc_bit    = 0;
   ui_RXCNT  = 0;
	
   SREG  &= 0x7F;	             //global INT aus                   
   GIMSK |= 0xC0;                //Ext Int enable
   MCUCR |= 0x0B;                //SDA L-trigger SCL H-trigger
   GIFR  = 0;                    //INT Flags ruecksetzen
   SREG  |= 0x80;				 //global INT an 
}


unsigned char i2cslave_readSDA(void)
{
   if ((PIND & 0x08) == 0) return(0);
    else return(1);
}


unsigned char i2cslave_readSCL(void)
{
   if ((PIND & 0x04) == 0) return(0);
    else return(1);
}


unsigned char i2cslave_RXTest(void)
{
   if ((state.bit_CONSTOP) && (ui_RXCNT > 0) && (c_RXBUFFER[0] == SLAVE_ADR))
     return(1);
   else return(0);
}


char * i2cslave_RXStr(char * pc_str)
{
   if (ui_RXCNT > 0) strcpy(pc_str,c_RXBUFFER+1); //lesen ohne Adresse
   ui_RXCNT = 0; 
   return(pc_str);
}

