//*****************************************************************************
//
//	Kommandointerpreter fuer UART0
//
//*****************************************************************************
#include <iom128v.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include "kommand0.h"
#include "watchdog.h"
#include "proto0.h"
#include "can.h"
#include "canreg.h"
#include "spibus.h"

char c_txstr[128];


//*****************************************************************************
//	Kommand - Kommandointerpreter
//  Input:	Befehlsstring
//	Output:	-
//*****************************************************************************
void Kommandointerpreter(char * pc_str)
{
unsigned char uc_ok = 0;

   watchdog_reset();
   if (strncmp(pc_str,"STAT",4) == 0)     uc_ok = BEFEHL_STATUS(pc_str+4); 
   if (strncmp(pc_str,"INT",3) == 0)      uc_ok = BEFEHL_INT(pc_str+3); 
   if (strncmp(pc_str,"OPEN",4) == 0)     uc_ok = BEFEHL_OPEN(pc_str+4); 
   if (strncmp(pc_str,"CLOSE",5) == 0)    uc_ok = BEFEHL_CLOSE(pc_str+5); 
   if (strncmp(pc_str,"WR",2) == 0)       uc_ok = BEFEHL_WR(pc_str+2); 
   if (strncmp(pc_str,"RD",2) == 0)       uc_ok = BEFEHL_RD(pc_str+2); 
   watchdog_reset();
   
   if (!uc_ok) SENDE_ERROR();
}


//*****************************************************************************
//	Kommand - Kommando Get Int
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_STATUS(char * pc_str)
{
char c_txt[16];
char c_string[10];
unsigned char uc_count,uc_bank;


  for(uc_bank=0;uc_bank < 8;uc_bank++)
  {
     watchdog_reset();  
     strcpy(c_txstr,"STATUS:");
  
    CAN_CS(1);
    SPI_Write(0x03);
    SPI_Write(uc_bank << 4);

    for(uc_count=0;uc_count < 16;uc_count++)
    {
      c_txt[uc_count] = SPI_Read(0x00);
      itoa(c_string,c_txt[uc_count],16);
	  if (c_txt[uc_count] < 0x10) strcat(c_txstr,"0");
      strcat(c_txstr,c_string);
	  strcat(c_txstr," ");
    }
  
    CAN_CS(0);
    sende_UART0(c_txstr);    
  }
  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Get Int
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_INT(char * pc_str)
{
char c_txt[16];

  watchdog_reset();  
  strcpy(c_txstr,"X:");
  c_txt[0] = 0x00;
  c_txt[1] = 0x00;
  strcat(c_txstr,c_txt);
  sende_UART0(c_txstr);  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Write Data
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_WR(char * pc_str)
{
CANMSG   MSG;
CANMSG * pMSG;

  MSG.can_ADR  = 0x00000002;
  MSG.uc_EXTID = CAN_INIT_TYPE_EX;
  MSG.uc_TYPE  = MSGTYPE_DATA;
  MSG.uc_len   = 0x08; 
  MSG.uc_DB0   = 0x54;
  MSG.uc_DB1   = 0x45;
  MSG.uc_DB2   = 0x53;
  MSG.uc_DB3   = 0x54;
  MSG.uc_DB4   = 0x00;
  MSG.uc_DB5   = 0x00;
  MSG.uc_DB6   = 0x00;
  MSG.uc_DB7   = 0x00;
  *pMSG = MSG;
  CAN_SENDMSG(pMSG,0);
  
  strcpy(c_txstr,"xWR");
  sende_UART0(c_txstr);  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando Read Data
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_RD(char * pc_str)
{
char c_txt[10];

  c_txt[0] = CAN_READREG(0x2c);
  c_txt[1] = 0x00;
  strcpy(c_txstr,"xRD:");
  strcat(c_txstr,c_txt);
  sende_UART0(c_txstr);  

//  sende_WertI(CAN_READREG(0x2c)); 
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando OPEN
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_OPEN(char * pc_str)
{
  strcpy(c_txstr,"CANOPEN");
  sende_UART0(c_txstr);  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Kommando CLOSE
//  Input:	-
//	Output:	-
//*****************************************************************************
unsigned char BEFEHL_CLOSE(char * pc_str)
{
  strcpy(c_txstr,"CANCLOSE");
  sende_UART0(c_txstr);  
  return(1);
}

 
//*****************************************************************************
//	Kommand - Sende String an PC
//  Input:	-
//	Output:	-
//*****************************************************************************
void SENDE_ERROR(void)
{
  watchdog_reset();
  strcpy(c_txstr,"ERROR");
  sende_UART0(c_txstr);
}


