//*****************************************************************************
//
//	Zeitgeber installieren 
//
//*****************************************************************************

#include <io8535.h>
#include "timer.h"
#include "8535int.h"

#pragma interrupt_handler timer0_int:TIM0_OVF

#define Zeitkonstante 100

unsigned char uc_toggle;
unsigned long ul_zeit;  //delay
unsigned long ul_z1;    //Abwaertszaehler1
unsigned long ul_z2;	//Abwaertszaehler2

//*****************************************************************************
//	Timer0-Interrupt
//  Input:	-
//	Output:	-
//*****************************************************************************

void timer0_int(void)
{
    TCNT0 = 0x06;	//256 - CNT250 1ms; UP-CNT
    TIFR &= 0x0FE;	//Overflow Flag reset

	if (uc_toggle)
	{
      uc_toggle = 0;
      if (ul_zeit > 0) ul_zeit--;
 	  if (ul_z1 > 0) ul_z1--;
      if (ul_z2 > 0) ul_z2--;
#ifdef timer_call	 
      timer_call();
#endif	
    }
	else uc_toggle = 1;
}


//*****************************************************************************
//	Timer - Init
//  Input:	-
//	Output:	-
//*****************************************************************************

void timer_init(void)
{
    SREG &= 0x7F;
 	ul_zeit = 0;
  				    //Timer init fuer 1ms = 4MHz / VT32 / CNT125 /2
    TIMSK |= 1;		//Timer0 Int enable
    TCCR0 = 0x02;	//keine Ausgaenge mit Timer schalten, Vorteiler div32
    TIFR &= 0x0FE;	//Overflow Flag reset
    SREG |= 0x80;	//global Int enable

}


//*****************************************************************************
//	Timer - DELAY
//  Input:	Zeitangabe (zeit in ms)
//	Output:	-
//*****************************************************************************

void timer_delay(unsigned long x)
{
 	 ul_zeit = x;
	 while(ul_zeit > 0);		  	   //warte bis zeit abgelaufen
}


//*****************************************************************************
//	Timer - CALL
//  Input:	-
//	Output:	-
//*****************************************************************************
//#ifndef timer_call
void timer_call(void)
{	        	 //leere Prozedure, kann in Hauptprogramm berschrieben werden
}
//#endif


//*****************************************************************************
//	Timer - Abwrtszaehler1
//  Input:	(zeit in ms)
//	Output:	-
//*****************************************************************************

void timer_setcount1(unsigned long x)
{
   ul_z1 = x;
}


//*****************************************************************************
//	Timer - Abwrtszaehler1
//  Input:	(zeit in ms)
//	Output:	-
//*****************************************************************************

unsigned long timer_getcount1(void)
{
   return(ul_z1);
}


//*****************************************************************************
//	Timer - Abwrtszaehler2
//  Input:	(zeit in ms)
//	Output:	-
//*****************************************************************************

void timer_setcount2(unsigned long x)
{
   ul_z2 = x;
}


//*****************************************************************************
//	Timer - Abwrtszaehler2
//  Input:	(zeit in ms)
//	Output:	-
//*****************************************************************************

unsigned long timer_getcount2(void)
{
   return(ul_z2);
}


