//*****************************************************************************
//
//	UART des ATmega Interrupt gesteuert
//
//*****************************************************************************
#include <io8535.h>
#include <string.h>
#include <stdlib.h>
#include "8535int.h"

#define BUF_LAENGE  50

#pragma interrupt_handler UART_RX_INT:UART_RXC
#pragma interrupt_handler UART_DRE_INT:UART_DRE

#define TEST   // enable CRC  => immer ok
#define BIIS   // enabel BIIS => ausgeschaltet 

char c_RXBUF[BUF_LAENGE];
char c_TXBUF[BUF_LAENGE];  
unsigned int ui_RX, ui_TX;

struct {
unsigned bit_RX_OVL_ERROR  	   : 1,
		 bit_RX_FRAME_ERROR    : 1,
		 bit_RX_RECEIVE        : 1,
		 bit_RX_RECEIVE_START  : 1,
		 bit_TX_TRANSMIT	   : 1,
		 bit_TX_TRANSMIT_START : 1,
		 bit_TX_FRAME_ERROR    : 1;
} bf; //bit_field



//*****************************************************************************
//	RX - INT
//*****************************************************************************

void UART_RX_INT(void)
{
unsigned char x;
         
    if ((USR & 0x10) == 0x10) 
	      bf.bit_RX_FRAME_ERROR = 1; 	 				//wenn Framing Error
	x = UDR;											//Daten einlesen
    if ((USR & 0x08) == 0x08) 
	      bf.bit_RX_OVL_ERROR = 1;						//wenn Overrun Error

    //Auswertung der Zeichen auf Gueltigkeit
   	if (x == '$') 
   	{
	   bf.bit_RX_RECEIVE_START = 1;			   //Stringanfang dedect
	   bf.bit_RX_FRAME_ERROR  = 0;				   //Errorflags reset
	   bf.bit_RX_OVL_ERROR    = 0;
	   bf.bit_RX_RECEIVE	   = 0;
	   ui_RX = 0;
   	}
   	
	if (bf.bit_RX_RECEIVE_START && (x == 0x0d))			 //Stringende dedect
	{                                                                      
		bf.bit_RX_RECEIVE_START = 0;			     //keine weiteren Zeichen lesen
		if (ui_RX > 0) c_RXBUF[ui_RX]=0;			//CR zu '\0' machen
		bf.bit_RX_RECEIVE       = 1;					//vollst. String anzeigen
	}    

	if (bf.bit_RX_RECEIVE_START && (x != 0x0A))
	{                       
	    c_RXBUF[ui_RX] = x;		
    	if (ui_RX < BUF_LAENGE)
	    {
	    	ui_RX +=1;
	    }
	    else {	
	   			ui_RX = 0;								//Daten verwerfen
   				bf.bit_RX_RECEIVE_START = 0;
	    	 }	
	}
    
	if (bf.bit_RX_FRAME_ERROR || bf.bit_RX_OVL_ERROR)			//wenn Fehler
	{												//es gab Uebertragungsfehler
		bf.bit_RX_RECEIVE_START = 0;					
		ui_RX = 0;
		bf.bit_RX_RECEIVE = 1;                      //Daten sind ungueltig, aber
    }											   //Errorflags sind gesetzt
}

//*****************************************************************************
//	TX - INT
//*****************************************************************************

void UART_DRE_INT(void)                 //TX-Interrupt
{
	if (bf.bit_TX_TRANSMIT && !bf.bit_TX_TRANSMIT_START)
	{
	 	bf.bit_TX_TRANSMIT_START = 1;				   //Uebertragung starten
	 	bf.bit_TX_FRAME_ERROR    = 0;				   //Fehler zuruecksetzen
	 	ui_TX = 0;	          
	}

	if (bf.bit_TX_TRANSMIT_START)
	{                     
		if (c_TXBUF[ui_TX] != 0)			  //gueltiges Zeichen im Sendepuffer
		{
		 	UDR = c_TXBUF[ui_TX];
		 	if (ui_TX < BUF_LAENGE)
		 	{
		 		ui_TX +=1;
		 	}
		 	else {                      
				 						//Bufferende und keine Endezeichen da
		 			bf.bit_TX_FRAME_ERROR    = 1; 
		 			bf.bit_TX_TRANSMIT       = 0;   
		 			bf.bit_TX_TRANSMIT_START = 0;
		    		UCR &= 0x0D8;					//UDRIE disable
		 		 }
		}
		else {										//Stringende dedect
				UDR = 0x0d;                         //Endekennung senden
				ui_TX = 0;
				bf.bit_TX_TRANSMIT_START = 0;
				bf.bit_TX_TRANSMIT		 = 0;
			 }
    }
    else {
    		UCR &= 0x0D8;							//UDRIE disable
    	 }	
}


//*****************************************************************************
//	UART - Initialisierung
//	Input:	-
//	Output:	-
//*****************************************************************************

void UART_INIT(void)
{
unsigned char x;
unsigned int i;

	SREG &= 0x7F;				//global INT aus                   
	ui_TX = 0;
	ui_RX = 0;
	bf.bit_RX_OVL_ERROR  	 = 0;
	bf.bit_RX_FRAME_ERROR    = 0;
	bf.bit_RX_RECEIVE        = 0;
	bf.bit_RX_RECEIVE_START  = 0;
	bf.bit_TX_TRANSMIT	     = 0;
	bf.bit_TX_TRANSMIT_START = 0;
	bf.bit_TX_FRAME_ERROR    = 0;
	
	for(i=0;i<255;i++)
	{
		c_TXBUF[i] = 0;
		c_RXBUF[i] = 0;
	}
	
	DDRD = (DDRD & 0x0FE) | 2;  //PE1 TXD, PE0 RXD
	PORTD = 0x00;				//Portausgang = 0                             
	UBRR = 0x19;				//9k6 Baudrate (25)
//	UCR = 0x18;					//Polling ,RX & TX der UART an
	UCR = 0x0B8;				//RX & TX der UART an, Interrupts enabled
	x = UDR;
	while((USR & 0x20) == 0);	//UDRE == 0 ?
	while((USR & 0x40) == 0x40);//TXC ==1 ?
	UDR = 0x0D;					//sende CR, damit TXC flag setzen
	SREG |= 0x80;				//global INT an
}


//*****************************************************************************
//	UART - Test, ob String empfangen
//	Input:	-
//	Output:	1 ja , 0 nein
//*****************************************************************************

unsigned int UART_RXTest(void)
{
 	if(bf.bit_RX_RECEIVE) return(1); 
    	 else return(0);
}


//*****************************************************************************
//  UART - CRC berechnen																	  
//  Input:  String (zwischen $ und * bzw. $ und \0 wird CRC berechnet)
//  Output: CRC16
//*****************************************************************************

unsigned int CrcBerechnen(char *pc_str)
{
  unsigned int ui_CRC = 0;
  unsigned int j = 0;
  unsigned char uc_crcflag = 0; // false
    // CRC berechnen (alle Zeichen zwischen $ und * bzw.
    //                alle Zeichen zwischen $ und \0)
    while (pc_str[j]) 
	{
      if (pc_str[j] == '$') 
	   {
         uc_crcflag = 1; //true
         j++;
       }
      if (pc_str[j] == '*')
         break;

      if (uc_crcflag) ui_CRC = ui_CRC ^ pc_str[j];
      j++;
    }
    return ui_CRC;
}


//*****************************************************************************
//  UART - CRC testen
//  Input:  String 
//  Output: 1 CRC
//*****************************************************************************

unsigned char CrcCheckKommando(char *pc_str)
{
  char * pc_stern;
  unsigned int ui_CRC;
  char c_temp[2];

  pc_stern = strchr(pc_str, '*');
    if (pc_stern)
	 {
		pc_stern++;
		strncpy(c_temp,pc_stern,2);

		if (c_temp[0] > 0x40) c_temp[0] =  c_temp[0] - 0x37;
		 else  c_temp[0] = c_temp[0] - 0x30;
		
		if (c_temp[1] > 0x40) c_temp[1] =  c_temp[1] - 0x37;
		 else  c_temp[1] = c_temp[1] - 0x30;
		
		ui_CRC = (c_temp[0] << 4) | c_temp[1];
		 
	    if (CrcBerechnen(pc_str) == ui_CRC) return(1); //true
	 } 		
	 
#ifdef TEST
    return(1);   //setze Ergebnis auf wahr
#endif

#ifndef TEST	 
    return(0); //false
#endif	
}


//*****************************************************************************
//  UART - Zeichen zu zwei Halbbyte
//  Input:  Zeichen
//  Output: String
//*****************************************************************************

char * MakeHalbbyte(char* pc_str,char c_byte)
{
  unsigned char c_x;

    c_x = (c_byte & 0xF0) >> 4;
	if ( c_x < 0x0A ) pc_str[0] = c_x + 0x30;
	  else pc_str[0] = c_x + 0x37;
	   
    c_x = (c_byte & 0x0F);
	if ( c_x < 0x0A ) pc_str[1] = c_x + 0x30;
	  else pc_str[1] = c_x + 0x37;
	
	pc_str[2]='\0'; 
    return(pc_str);
}


//*****************************************************************************
//  UART - CRC anhngen																	  
//  Input:  String ohne CRC
//  Output: String mit angehaengter CRC
//*****************************************************************************

char * CrcAnhaengen(char *pc_dst, char *pc_src)
{
  char c_temp[3] ="";
  unsigned int ui_CRC;
  
	strcpy(pc_dst,pc_src);
	strcat(pc_dst,"*");
	ui_CRC = CrcBerechnen(pc_src);
    strcat(pc_dst,MakeHalbbyte(c_temp,(char) ui_CRC ));
	return pc_dst;
}


//*****************************************************************************
//  UART - CRC entfernen																	  
//  Input:  String mit CRC
//  Output: String ab CRC eingekuerzt
//*****************************************************************************

char * CrcEntfernen(char *pc_src)
{
  char * pc_stern;
  
    pc_stern = strchr(pc_src, '*');
	if (pc_stern != NULL) pc_src[strlen(pc_src) - strlen(pc_stern)] = '\0';
    return pc_src;
}




//*****************************************************************************
//	UART - String senden
//	Input:	char * Sendestring
//	Output:	-
//*****************************************************************************

void UART_TXString(char * pc_str)
{
  char c_temp[BUF_LAENGE];
   
	if (strlen(pc_str) > 0)
	 {
    	CrcAnhaengen(c_temp,pc_str);
     }
	 else strcpy(c_temp,""); 
	while(bf.bit_TX_TRANSMIT);		//warte bis Sendepuffer leer
	strcpy(c_TXBUF,c_temp);
	bf.bit_TX_TRANSMIT = 1;		//Start der Uebertragung
	UCR |= 0x20;				//UDRIE enable  	
}


//*****************************************************************************
//	UART - String senden
//	Input:	char * Sendestring
//	Output:	-
//*****************************************************************************

void UART_TXStr(char * pc_str)
{
  char c_temp[BUF_LAENGE];
   
	if (strlen(pc_str) > 0)
	 {
    	strcpy(c_temp,pc_str);
     }
	 else strcpy(c_temp,""); 
	while(bf.bit_TX_TRANSMIT);		//warte bis Sendepuffer leer
	strcpy(c_TXBUF,c_temp);
	bf.bit_TX_TRANSMIT = 1;		//Start der Uebertragung
	UCR |= 0x20;				//UDRIE enable  	
}



//*****************************************************************************
//	UART - String empfangen
//	Input:	char * Empfangsstring (freier Speicher)
//	Output:	-
//*****************************************************************************

char * UART_RXString(char * pc_str)
{      
  char c_temp[BUF_LAENGE];
           
    while(!bf.bit_RX_RECEIVE);		//warte auf String
	if (bf.bit_RX_FRAME_ERROR || bf.bit_RX_OVL_ERROR)
	{
	   pc_str = '\0';
	}
	else {
      		strcpy(c_temp,c_RXBUF);
			if (CrcCheckKommando(c_temp) )  //wenn CRC gltig
        	 {
	   	       CrcEntfernen(c_temp);
			   strcpy(pc_str,c_temp);	        	
             }
		     else pc_str = '\0';		
	     }	
	bf.bit_RX_RECEIVE = 0;		//String gelesen

 	return(pc_str);
}



//*****************************************************************************
//	UART - String empfangen
//	Input:	char * Empfangsstring (freier Speicher)
//	Output:	-
//*****************************************************************************

char * UART_RXStr(char * pc_str)
{      
  char c_temp[BUF_LAENGE];
           
    while(!bf.bit_RX_RECEIVE);		//warte auf String
	if (bf.bit_RX_FRAME_ERROR || bf.bit_RX_OVL_ERROR)
	{
	   pc_str = '\0';
	}
	else {
      		strcpy(pc_str,c_RXBUF);
	     }	
	bf.bit_RX_RECEIVE = 0;		//String gelesen

 	return(pc_str);
}


